
    
    
    PROCESSOR 10LF320    

    #include "xc.inc"
    #include "pic10lf320.inc"

    

    ;__CONFIG        _BODEN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _XT_OSC
  CONFIG  FOSC = INTOSC             ; Oscillator Selection bits (XT oscillator: Crystal/resonator on RA6/OSC2/CLKOUT and RA7/OSC1/CLKIN)
  CONFIG  WDTE = ON             ; Watchdog Timer Enable bit (WDT enabled)
  CONFIG  PWRTE = OFF           ; Power-up Timer Enable bit (PWRT disabled)
  CONFIG  MCLRE = ON            ; RA5/MCLR pin function select (RA5/MCLR pin function is MCLR)
  CONFIG  BOREN = ON            ; Brown-out Reset Enable bit (BOD Reset enabled)
  CONFIG  LVP = ON              ; Low-Voltage Programming Enable bit (RB4/PGM pin has PGM function, low-voltage programming enabled)
  ;CONFIG  CPD = OFF             ; Data Code Protection bit (Data memory code protection off)
  CONFIG  CP = OFF              ; Code Protection bits (Program memory code protection off)
  
  
 LINE_OUTA EQU 0x40 ;Address which this to be printed to screen
 LINE_OUTB EQU 0x41 ;Address which this to be printed to screen
 LINE_COUNTER EQU 0x42 ;Address which this to be printed to screen
 VIDEO_FLAGS EQU 0x43   ;Flags I'll use later, haven't decided yet
                        ;Bit 0 : carrier flag, if high draw to screen
                        ;Bit 0 : carrier flag, if high new frame
                        ;Bit 7 : new frame
 
 FRAMES             EQU 0x44    ;Count frames
 FRAMES_PER_STEP    EQU 2       ;Number of frames, in powers of 2 (select the bit)
                        
 INTERRUPT_W EQU 0x45
 INTERRUPT_STATUS EQU 0x46
 INTERRUPT_FSR EQU 0x47
 
 HL_SHIFT_COUNTER EQU 0x48 ;Count how many times the LINE_OUT register has been shifted
 
 TEMP EQU 0x4A ;generic processing address
 COUNTER EQU 0x4B ;generic loop counter
 COUNTER2 EQU 0x4C
 DELAY_COUNTER EQU 0x4D ;Counter for the delay route
 ROTATE_FLAGS EQU  0x4E ;Flags for rota
 
 SEED   EQU     0x4F ;Random Seed storage
 
 
 ARRAY EQU 0x5E ; Main image grid
 STRIP EQU 0x50 ; 3 lines plus output on 4th
  
  
    
    PSECT res_vect, class=CODE, delta=2
    res_vect:
        GOTO R
    PSECT Int_vect, class=CODE, delta=2
    Int_vect:
        GOTO INTERRUPT



R:
			    ;Disable pull-ups
			    ;INT on rising edge
			    ;TMR0 to CLKOUT
			    ;TMR0 Incr low2high trans.
			    ;Prescaler assign to WDT
			    ;Prescaler rate is 1:16
    MOVLW   0b11010001	;Set PIC options (See datasheet).
    MOVWF   OPTION_REG	;Write the OPTION register.
    
    MOVLW   0b01110000 ;Set Osc to 16MHz
    MOVWF   OSCCON
    ;
    
    ;MOVLW   0b0000000	;all RA ports are outputs
    CLRF   TRISA
    
    MOVLW   0b00011110
    MOVWF   WDTCON
    
    ;Need to set the oscillator frequency here
    
    ;intilized glider an blicker
    ;........
    ;........
    ;...#....
    ;....#...
    ;..###...
    ;........
    ;........
    ;........
    ;MOVLW   0b00000000
    ;MOVWF   ARRAY
    ;MOVLW   0b00000000
    ;MOVWF   ARRAY+1
    
    
    MOVLW   ARRAY
    MOVWF   FSR
    
FILL_ARRAY:
    ;Here the array is Fill with a poorly written pseudorandom noise
    ;The watchdog timing runs on a different clock (I think) so is a good source of entropy
    ;Plus the initial state of the memory on boot up is slightly random
    MOVF    TMR0, 0
    ADDWF   SEED, 1
    RRF     SEED, 0
    XORWF   SEED, 0
    XORLW   0b11000110 ;probably a better value to put here, just a guess
    XORWF   FSR , 0
    
    MOVWF   INDF
    
    INCF    FSR
    BTFSS   FSR, 7
    GOTO    FILL_ARRAY
    
    
    ;Don't need this?
    ;MOVLW   208             ;Int counter such that on (256-208) + 256 = 304 lines
    ;MOVWF   LINE_COUNTER
    
    MOVLW   0b10100000
    MOVWF   INTCON		;Enable global internal and timer0 inteerupt
    
    
    ;Copy lines, trying to implenent torus topology so first line is next to last
    ;MOVF    0x67, 0
    ;MOVWF   0x50
    ;MOVF    0x60, 0
    ;MOVWF   0x51
    ;MOVF    0x61, 0
    ;MOVWF   0x52
    ; Output of this seltion saved in 0x53, save ARRAY[0] to 0x54 for loop
    
    GOTO MAIN
    
    

;Main loop
;Simply waits for a few frames before calculating the next interation

MAIN:
    
    BTFSS FRAMES, FRAMES_PER_STEP
    GOTO MAIN
    
    CLRF    FRAMES
    
    CALL STEP
    
    GOTO    MAIN

    
    
;############ Conway's game of life processing ################
;3 rows are copied to a temp array and processed there

STEP:
    MOVLW 0b00010000 ;Start counting from 16 which is the number of rows
    MOVWF COUNTER2
    
    MOVF ARRAY+0,0
    MOVWF ARRAY+16 ; save previuos row here
    
    MOVF ARRAY+17,0
    MOVWF ARRAY+33 ; save previuos row here
    
    MOVF    ARRAY+15, 0
    MOVWF   STRIP
    MOVF    ARRAY+0, 0
    MOVWF   STRIP+1
    MOVF    ARRAY+1, 0
    MOVWF   STRIP+2
    
    MOVF    ARRAY+32, 0
    MOVWF   STRIP+4
    MOVF    ARRAY+17, 0
    MOVWF   STRIP+5
    MOVF    ARRAY+18, 0
    MOVWF   STRIP+6
    
    
    MOVLW   ARRAY+1
    MOVWF   FSR
    
    
PROC_LINE:
    MOVLW 0b00010000
    MOVWF COUNTER ;load 8 into counter
    CLRF STRIP+3
    CLRF STRIP+7
COUNT:
    ;count neighboor cells
    CLRF TEMP
    BTFSC STRIP, 2
    INCF TEMP
    BTFSC STRIP, 1
    INCF TEMP
    BTFSC STRIP, 0
    INCF TEMP
    BTFSC STRIP+1, 2
    INCF TEMP
;    BTFSC STRIP+1, 1 ; center cell, don't count
;    INCF TEMP
    BTFSC STRIP+1, 0
    INCF TEMP
    BTFSC STRIP+2, 2
    INCF TEMP
    BTFSC STRIP+2, 1
    INCF TEMP
    BTFSC STRIP+2, 0
    INCF TEMP
    
    ; apply rule, a little bit of bit checking, faster and less code than explicit equality checking for rules
    BTFSS STRIP+1, 1
    GOTO DEAD
;ALIVE
    BTFSC TEMP, 1
    BSF STRIP+3 , 1
    BTFSC TEMP, 2
    BCF STRIP+3 , 1
    BTFSC TEMP, 3
    BCF STRIP+3 , 1
    GOTO CELL_SET
DEAD:
    BSF STRIP+3 , 1
    DECF TEMP
    DECF TEMP
    DECFSZ TEMP
    BCF STRIP+3 , 1
CELL_SET:
    ;Rotate the bytes through each other
    ; for example row 1 is 16 cell across so is made of 2 bytes
    ; when I rotate through carry byte 1 and the byte 2 I've rotated
    ; the entire 16 cells over 1 cell. A little extra code is needed
    ; to get the last bit in the first position.
    ; By doing this the same code above is reused again for the same
    ; memory address but different cells.
    
    CLRF    ROTATE_FLAGS
    
    BCF     STATUS, 0
    
    RRF     STRIP+0
    RRF     STRIP+4
    BTFSC   STATUS, 0
    BSF     STRIP+0, 7
    
    BCF     STATUS, 0
    
    RRF     STRIP+1
    RRF     STRIP+5
    BTFSC   STATUS, 0
    BSF     STRIP+1, 7
    
    BCF     STATUS, 0
    
    RRF     STRIP+2
    RRF     STRIP+6
    BTFSC   STATUS, 0
    BSF     STRIP+2, 7
    
    BCF     STATUS, 0
    
    RRF     STRIP+3
    RRF     STRIP+7
    BTFSC   STATUS, 0
    BSF     STRIP+3, 7
    

    
    DECFSZ COUNTER
    GOTO COUNT
    ;##### Shift to next row #####
    
    
    MOVF STRIP+1,0
    MOVWF STRIP+0
    MOVF STRIP+2,0
    MOVWF STRIP+1
    
    MOVF STRIP+5,0
    MOVWF STRIP+4
    MOVF STRIP+6,0
    MOVWF STRIP+5
    
    ;Write STRIP out to ARRAY at FSR location
    DECF FSR
    MOVF STRIP+3, 0
    MOVWF INDF
    INCF FSR
    
    INCF FSR
    MOVF INDF, 0
    MOVWF STRIP+2
    
    MOVLW   16
    ADDWF   FSR,1
    
    DECF FSR
    MOVF STRIP+7, 0
    MOVWF INDF
    INCF FSR
    
    INCF FSR
    MOVF INDF, 0
    MOVWF STRIP+6
    
    MOVLW   17
    SUBWF   FSR,1
    
    DECFSZ COUNTER2
    GOTO PROC_LINE
    
    ;Might need to add a horizontal/Vertical sync wait here somewhere
    
    RETURN


    
;############ Composite video out section ################

    
INTERRUPT:
    CLRF    PORTA
    BCF     TRISA, 1 ; Set pin RC1 low
    
    ;Saving the important registers here
    MOVWF INTERRUPT_W

    MOVF STATUS, 0
    MOVWF INTERRUPT_STATUS
    
    MOVF FSR, 0
    MOVWF INTERRUPT_FSR
    
    
    INCF    LINE_COUNTER,1
    BTFSC   STATUS,2
    INCF    VIDEO_FLAGS,1
    NOP

    ;delay for rest of horizontal syncing interval
    MOVLW   1
    CALL    DELAY
    
    BSF     TRISA, 1 ;Set to high impedence mode
    BCF	    INTCON , 2 ;reset timer for next horizontal sync
    
    ;Set timing for next horizontal sync
    ;NOP
    ;NOP
    MOVLW 202 ;Quick guess
    MOVWF TMR0
    
    BTFSC VIDEO_FLAGS,1 ; Vertical sync
    CALL VERT_SYNC
    
    BTFSC VIDEO_FLAGS,0 ; Vertical sync
    CALL HOR_LINE
    
    ;
    
INT_RETURN:
    ;Reloading the important registers
    MOVF    INTERRUPT_FSR,0
    MOVWF   FSR
    
    MOVF    INTERRUPT_STATUS,0
    MOVWF   STATUS
    
    MOVF    INTERRUPT_W,0

    RETFIE
    
    
HOR_LINE:
    ; must be 64us long exactly, I think
    ; Takes a memory address as output
    ; Includes the horizontal sync
    
    BSF     PORTA, 1
    
    MOVLW   4
    CAll DELAY
    
    SWAPF   LINE_COUNTER,0
    ANDLW   0b00001111
    ADDLW   ARRAY
    
    MOVWF   FSR
    MOVF    INDF,0
    MOVWF   LINE_OUTA
    
    MOVLW   17
    ADDWF   FSR,1
    MOVF    INDF,0
    MOVWF   LINE_OUTB
    
    MOVLW   16
    MOVWF   HL_SHIFT_COUNTER
    
    BCF     STATUS, 0  ;Can maybe assume status carry bit always zero here?
    BTFSC   LINE_OUTB, 0
    BSF     STATUS, 0
HOR_LINE_SHIFT:
    
    MOVF    LINE_OUTB, 0
    ANDLW   0b0000010
    MOVWF   TRISA
    
    ;BTFSS   LINE_OUTA,7
    ;BSF     TRISA, 1
    ;BTFSC   LINE_OUTA,7
    ;BCF     TRISA, 1
    
    RRF     LINE_OUTA
    RRF     LINE_OUTB
    ;MOVLW 1
    ;CALL DELAY
    
    DECFSZ  HL_SHIFT_COUNTER
    GOTO    HOR_LINE_SHIFT
    
    
    BSF     TRISA, 1
    CLRF    PORTA
    
    
    RETURN


    
VERT_SYNC:
    ; Vertical sync timing sequence, a lot of code here that could be shorten... maybe?
    BCF     INTCON, 5
    
    CLRF    PORTA
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
   
    CALL    LONG_SYNC
    
    CALL    LONG_SYNC
    
    CALL    LONG_SYNC
    
    CALL    LONG_SYNC
    
    CALL    LONG_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    CALL    SHORT_SYNC
    
    NOP
    NOP
    BCF     TRISA, 1
    
    MOVLW   208
    MOVWF   LINE_COUNTER
    CLRF    VIDEO_FLAGS
    INCF    FRAMES
    
    NOP
    NOP
    NOP
    
    BSF     TRISA, 1
    

    ;Do first Horizontal sync?
    
    MOVLW   118
    MOVWF   TMR0
    
    BCF	    INTCON , 2
    BSF     INTCON, 5
    
    RETURN

LONG_SYNC:
    BCF     TRISA, 1
    MOVLW   38
    CALL    DELAY
    BSF     TRISA, 1
    NOP
    NOP
    NOP
    RETURN
    
SHORT_SYNC:
    BCF     TRISA, 1
    NOP
    NOP
    NOP
    NOP
    NOP
    NOP
    NOP
    BSF     TRISA, 1
    MOVLW   36
    NOP
    NOP
    CALL    DELAY
    RETURN
    
;############ Utility functions ################

    
    ;Delay function, will delay by 4+3W operations. Not including the operation to set the W register before calling
DELAY:
    MOVWF DELAY_COUNTER
DELAY_LOOP:
    DECFSZ DELAY_COUNTER
    GOTO DELAY_LOOP
    RETURN


    
    END 